/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2001, 2005 NoMachine, http://www.nomachine.com.          */
/*                                                                        */
/* NXUEXEC, NX protocol compression and NX extensions to this software    */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/


boolean dirNotWritableForEveryone(const char *dirname){
  struct stat dirstat;
    if (lstat (dirname,&dirstat) < 0) {
      fprintf(stderr,"Error: Cannot stat '%s'. Error is %d '%s'.\n",dirname,errno,strerror(errno));
      exit(1);
    }
    if (! (S_ISDIR (dirstat.st_mode))) {
      fprintf(stderr,"Error: '%s' is not a directory.\n",dirname);
      exit(1);
    }
    
    if (S_IWOTH & dirstat.st_mode) {
      return false;
    }
    else {
      return true;
    }
}

boolean dirExist(const char *dirname) {
  struct stat dirstat;
  if (lstat (dirname,&dirstat) < 0) {
    fprintf(stderr,"Error: Cannot stat '%s'. Error is %d '%s'.\n",dirname,errno,strerror(errno));
    exit(1);
  }
  return true;
}

boolean scriptNotWritableForEveryone(const char *filename) {
   struct stat filestat;
   if (lstat (filename,&filestat) < 0) {
     fprintf(stderr,"Error: Cannot stat '%s': Error is %d '%s'.\n",filename,errno,strerror(errno));
     exit(1);
   }
   if (S_IWOTH & filestat.st_mode) {
      return false;
    }
    else {
      return true;
    }
}

boolean scriptExist(const char *filename) {
  struct stat filestat;
  if (lstat (filename,&filestat) == 0) {
    return true; 
  }
  else {
    fprintf(stderr,"Error: Cannot stat '%s': Error is %d '%s'.\n",filename,errno,strerror(errno));
    exit(1);
  }
}



boolean scriptIsFile(const char *filename) {
  struct stat filestat;
  if (lstat (filename,&filestat) < 0) {
    fprintf(stderr,"Error: Cannot stat '%s': Error is %d '%s'.\n",filename,errno,strerror(errno));
    exit(1);
  }
  else {
    if (S_ISREG(filestat.st_mode)) return true;
    else return false;
  }
  
}

ino_t getInode(const char *dirname) {
  struct stat dirstat;
  if (stat (dirname,&dirstat) < 0) {
    fprintf(stderr,"Error: Cannot stat '%s': Error is %d '%s'.\n",dirname,errno,strerror(errno));
    exit(1);
  }
  return dirstat.st_ino;
}

boolean checkPath(const char *scriptname) {
  char *dirc, *basec, *bname, *dname;
  extern char correctScriptName [];
  extern char NXSDIR [];
  dirc = strdup(scriptname);
  basec = strdup(scriptname);
  dname = dirname(dirc);
  if (strcmp(dname,".")) {
    if (getInode(dname) != getInode(NXSDIR)) {
      fprintf(stderr,"Error: Cannot run scripts from '%s' directory.\n",dname);
      fprintf(stderr,"Error: The script has to be in '%s' directory.\n",NXSDIR);
      exit(1);
    }
  }
    else {
      bname = basename(basec);
      if ((strlen(NXSDIR)+1+strlen(bname))>MAX_LENGTH) {
        fprintf(stderr,"Error: Full path too long. More than '%d' for '%s/%s.\n",MAX_LENGTH,NXSDIR,bname);
	exit(1);
      }
      strcpy(correctScriptName,NXSDIR);
      strcat(correctScriptName,"/");
      strcat(correctScriptName,bname);
  }
  
  return true;
}

void checkArg(int argc, char *argv[]){

  int x;
  for(x=0;x<argc;x++){
    if(strchr(argv[x],';')){
      fprintf(stderr,"Error: Wrong character ';' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'&')){
      fprintf(stderr,"Error: Wrong character '&' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'|')){
      fprintf(stderr,"Error: Wrong character '|' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'>')){
      fprintf(stderr,"Error: Wrong character '>' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'<')){
      fprintf(stderr,"Error: Wrong character '<' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'*')){
      fprintf(stderr,"Error: Wrong character '*' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'?')){
      fprintf(stderr,"Error: Wrong character '?' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'`')){
      fprintf(stderr,"Error: Wrong character '`' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'$')){
      fprintf(stderr,"Error: Wrong character '$' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'(')){
      fprintf(stderr,"Error: Wrong character '(' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],')')){
      fprintf(stderr,"Error: Wrong character ')' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'{')){
      fprintf(stderr,"Error: Wrong character '{' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'}')){
      fprintf(stderr,"Error: Wrong character '}' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'[')){
      fprintf(stderr,"Error: Wrong character '[' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],']')){
      fprintf(stderr,"Error: Wrong character ']' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'!')){
      fprintf(stderr,"Error: Wrong character '!' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
    if(strchr(argv[x],'#')){
      fprintf(stderr,"Error: Wrong character '#' in argument: '%s'.\n",argv[x]);
      exit(1);
    }
  }
  return;
}

char *getUser(){
  uid_t uid;
  struct passwd *pw;
  
  uid = getuid();
  pw = getpwuid(uid);
  return pw->pw_name;
}	  
