/*
   nxuexec: Run a script suid/sgid on local node.
   Usage: ./nxuexec scriptname
   Version 1.5.0-1

*/

/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2001, 2005 NoMachine, http://www.nomachine.com.          */
/*                                                                        */
/* NXUEXEC, NX protocol compression and NX extensions to this software    */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdarg.h>
#include <libgen.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <pwd.h>

#define EGET()  (errno)
#define CONFIGFILE "/usr/NX/etc/node.cfg"
#define DIRCONFIGFIELD "NXSDIR"
#define PRENXSDIR "/usr/NX/scripts/restricted"
#define FILECOMMAND "/usr/bin/file"
#define VERSION "1.5.0-2"
#define MAX_LENGTH 255
#define progname "nxuexec"
#ifndef boolean
#define boolean int
#endif

#if !defined(false) || (false != 0)
#define false   0
#endif

#if !defined(true) || (true != 0)
#define true    1
#endif

#if defined(DEBUG)
#undef DEBUG 
#endif

ssize_t nxgetline(char **bufferp,size_t *leng,FILE *f);
boolean dirNotWritableForEveryone(const char *dirname);
boolean scriptNotWritableForEveryone(const char *filename);
boolean scriptExist(const char *filename);
boolean scriptTextScript(const char *filename);
boolean scriptSuided(const char *filename);
boolean scriptGuided(const char *filename);
uid_t getUidOwnerFile(const char *filename);
gid_t getGidOwnerFile(const char *filename);
boolean dirExist(const char *dirname);
boolean checkPath(const char *scriptname);
boolean setNXSDIR();

char pathtodir [255];
char NXSDIR [255];
char correctScriptName [255];
#include "functions.c"
#include "functionsSuid.c"
#include "parseConfig.c"
#include "checkScript.c"
#include "nxgetline.c"


int main (int argc,char *argv[], char **envp) {
  uid_t executeUid;
  gid_t executeGid;
  int result;
  char *scriptname;
  char *username;
    
  if (argc <= 1) {
    fprintf(stderr,"NXUEXEC - %s \n\n",VERSION);
    fprintf(stderr,"Usage: ./%s [name_of_script]\n",progname);
    fprintf(stderr,"Runs a script.\n");
    fprintf(stderr,"The script is searched for into the '%s'\n",PRENXSDIR);
    fprintf(stderr,"directory. The directory name can be changed by setting the\n");
    fprintf(stderr,"parameter '%s' in the '%s' config file.\n",DIRCONFIGFIELD,CONFIGFILE);

    exit(1);
  }

  checkArg(argc,argv);

  setNXSDIR();

  username=getUser();

  if (strncmp(pathtodir,"",1)) {
    if (strlen(pathtodir) < 254 ) {
      strcpy(NXSDIR,pathtodir);
    }
    else {
      fprintf(stderr,"Error: path '%s' is too long.\n",pathtodir);
      exit(1);
    }
  }
  else {
    #ifdef DEBUG
      fprintf(stdout,"Info: Setting '%s' for NX scripts directory.\n",PRENXSDIR);
    #endif
    if (strlen(PRENXSDIR) < 254) {
      strcpy(NXSDIR,PRENXSDIR);
    }
    else {
      fprintf(stderr,"Error: path '%s' is too long.\n",PRENXSDIR);
      exit(1);
    }
  }
  
  if (! dirNotWritableForEveryone(NXSDIR)) {
    fprintf(stderr,"Error: Directory '%s' is writable for everyone.\n",NXSDIR);
    exit(1);
  }

  scriptname = argv[1];
  
  if (! checkPath(scriptname)) {
    fprintf(stderr,"Error: Filename or path not correct '%s'.\n",scriptname);
    exit(1);
  }
  
  if (correctScriptName[0] != 0) {
    scriptname=correctScriptName;
  }
  
  if (! scriptExist(scriptname)) {
    fprintf(stderr,"Error: Cannot open '%s':\n%s.\n",scriptname,strerror(errno));
    exit(1);
  }
  
  if (! scriptIsFile(scriptname)) {
    fprintf(stdout,"Error: '%s' is not a regular file.\n",scriptname);
    exit(1);
  }
   
  if (! scriptNotWritableForEveryone(scriptname)) {
    fprintf(stderr,"Error: File '%s' is writable for everyone.\n",scriptname);
    exit(1);
  }
  
  if (! scriptTextScript(scriptname)) {
    fprintf(stderr,"Error: File '%s' is not a text script.\n",scriptname);
    exit(1);
  }
  #ifdef DEBUG
    fprintf(stdout,"Info: Runnig with: uid[%d] euid[%d] gid[%d] egid[%d].\n",getuid(),geteuid(),getgid(),getegid());
  #endif
 
  if (scriptSuided(scriptname)) {
     executeUid=getUidOwnerFile(scriptname);
    #ifdef DEBUG
      fprintf(stdout,"Info: Script is suided. Set executeUid for executions on '%d'.\n",executeUid);
    #endif
  }
  else {
     executeUid=getuid();
    #ifdef DEBUG
      fprintf(stdout,"Info: Script is not suided. Set executeUid for executions on '%d'.\n",executeUid);
    #endif
  }
  if (scriptGuided(scriptname)) {
     executeGid=getGidOwnerFile(scriptname);
    #ifdef DEBUG
      fprintf(stdout,"Info: Script have group id set. Set executeGid for executions on '%d'.\n",executeGid);
    #endif
  }
  else {
    executeGid=getgid();
    #ifdef DEBUG
      fprintf(stdout,"Info: Script is not guided. Set executeGid for executions on '%d'.\n",executeGid);
    #endif
  }
  #ifdef DEBUG
    fprintf(stdout,"Info: Executing with: uid [%d] gid [%d] \n",executeUid,executeGid); 
  #endif

  
  result = execute(scriptname,executeUid,executeGid,argc,argv,username);
  result = result >> 8;
  #ifdef DEBUG
    fprintf(stderr,"Im terminating with result '%d'.\n",result);
  #endif
  exit(result);
}

int execute (const char *scriptname,uid_t executeUid,gid_t executeGid, int argc, char *argv[], char *username) {
  int pid,status;
  extern char **environ;
  //char *argv[3];
  int options;
  int x;

  if (scriptname == NULL) {
    fprintf(stderr,"Error: scriptname cannot be NULL.\n");
    return 1;
  }

/*  pid = fork();
  if (pid == -1) {
    fprintf(stderr,"Error: Cannot fork: '%s'.\n",strerror(errno));
    return -1;
  }

  if (pid == 0) {
*/
    setgid(executeGid);
    if (errno == -1) {
      fprintf(stderr,"Error: Cannot set setgid '%d': %s.\n",executeGid,strerror(errno));
      exit(1);
    }
    #ifdef DEBUG
      fprintf(stdout,"Info: executive gid set to '%d'.\n",executeGid);
      fprintf(stdout,"Info: real gid [%d] egid [%d].\n",getgid(),getegid());
    #endif

    setuid(executeUid);
    if (errno == -1) {
      fprintf(stderr,"Error: Cannot set uid '%d': %s.\n",executeUid,strerror(errno));
      exit(1);
    }
    #ifdef DEBUG
      fprintf(stdout,"Info: executive uid set to '%d'.\n",executeUid);
      fprintf(stdout,"Info: real uid [%d] euid [%d].\n",getuid(),geteuid());
      fprintf(stdout,"Info: real gid [%d] egid [%d].\n",getgid(),getegid());
    #endif

    putenv("PATH=''");
    putenv("IFS=' \t\n'");
    #ifdef __sun
    putenv("CDPATH=");
    putenv("ENV=");
    putenv("BASH_ENV=");
    putenv("LD_LIBRARY_PATH=");
    putenv("LD_PRELOAD=");
    putenv("LD_AOUT_LIBRARY_PATH=");
    putenv("LD_AOUT_PRELOAD=");
    putenv("LD_KEEPDIR=");
    #else
    unsetenv("CDPATH");
    unsetenv("ENV");
    unsetenv("BASH_ENV");
    unsetenv("LD_LIBRARY_PATH");
    unsetenv("LD_PRELOAD");
    unsetenv("LD_AOUT_LIBRARY_PATH");
    unsetenv("LD_AOUT_PRELOAD");
    unsetenv("LD_KEEPDIR");
    #endif

    argv[0] = (char *) scriptname;
    if(argc>2){
      argv[1] = username;
      for(x=2;x<argc;x++){
        argv[x]=argv[x];
      } 
    }
    else{
      argv[1] = NULL;
    }

    execve ((char *) scriptname,argv,environ);
    if (errno != 0) {
      fprintf(stderr,"Error: Cannot execute '%s': %s.\n",scriptname,strerror(errno));
      exit(errno);
    }
    #ifdef DEBUG
      fprintf(stderr,"Terminating with 0.\n",errno,strerror(errno));
    #endif
    exit(0);
}
